<?php

/*
Copyright (C) 2001-2005
by Justin Woods <justin@woodsmedia.com>
and David Sowder <david@sowder.edu>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/

// $Id: webstamp.inc,v 1.32 2005/05/22 22:57:47 davids Exp $

function get_needed_tags_list($workingset) {
  preg_match_all( "/\{(=?_?[A-Z][^}]+)\}/", $workingset, $m );
  $m = $m[ 1 ];
  if( !is_array( $m )) {
    return array();
  }
  reset( $m );
  $result = array();
  while( list( $k, $v ) = each( $m )) {
    if( !isset( $result[ $v ] )) {
      $result[ $v ] = $v;
    }
  }
  if( count( $result )) {
    return $result;
  } else {
    return array();
  }
}

function get_directory_steps_to_libs_dir( $directory ) {
  $dir_list = array();
  while(  $directory = dirname( $directory )) {
    $dir_list[] = $directory;
    if( $directory == '/' ) {
      break;
    }
    else if( file_exists( $directory . '/.libs' )) {
      $dir_list[] = $directory . '/.libs';
      break;
    }
  }
  return array_reverse( $dir_list );
}

function get_reldir_matched_url( $reldir,$url ) {
  $reldir = $reldir . '/';
  if( !strncmp( $reldir, './', 2 )) {
    $reldir = substr( $reldir, 2 );
  }
  while( !strncmp( $reldir, '../', 3 )) {
    $reldir = substr( $reldir, 3 );
    $url = dirname( $url ) . '/';
  }
  if( !strncmp( $url, '//', 2 )) {
    $url = substr( $url, 1 );
  }
  return $url;
}

function get_file_list( $directory, $prefix = '' ) {
  global $OURLIBS, $WEBSTAMP_TAGFILES_FILE_NAME;

  # Start with an empty file_list array
  $file_list = array();
  # Add the tagfiles from the given directory in the array
  $file_list = get_file_list_helper( $file_list, $directory, $prefix );
  # Add the tagfiles from the OURLIBS directory in the array
  $file_list = get_file_list_helper( $file_list, $OURLIBS, '=' );

  # **FIXME** Remove call to the old implementation used to compare results during new-style implementation
  #$file_list_old = get_file_list_old( $directory, $prefix );

  #echo "DEBUG: file_list BEGIN<br>\n";
  #print_r( $file_list );
  #echo "<br>\nDEBUG: file_list END<br>\n";

  #echo "DEBUG: file_list_old BEGIN<br>\n";
  #print_r( $file_list_old );
  #echo "<br>\nDEBUG: file_list_old END<br>\n";

  return $file_list;
}

function get_file_list_helper( $file_list, $directory, $prefix ) {
  global $WEBSTAMP_TAGFILES_FILE_NAME;

  #echo "DEBUG: directory: [${directory}]<br>\n";
  #echo "DEBUG: prefix: [${prefix}]<br>\n";
  # Get the Webstamp tagfiles file from the given directory into an array,
  $webstamp_tagfiles_file_lines = file( $directory . '/' . $WEBSTAMP_TAGFILES_FILE_NAME );
  if( false === $webstamp_tagfiles_file_lines ) {
    # The Webstamp tagfiles file does not exist or is not readable with the current permissions
    die( '<b>"webstamp webify" has not been run on the ' . "'" . realpath( $directory ) . "'" . ' directory.  We cannot continue Webstamp processing for this page until that has been completed.</b>' );
  }
  # Iterate over lines, importing variables into file_list
  foreach ($webstamp_tagfiles_file_lines as $webstamp_tagfiles_file_line) {
    $webstamp_tagfiles_file_line = trim( $webstamp_tagfiles_file_line );
    # Check for a comment in the line
    $pound_pos = strpos( $webstamp_tagfiles_file_line, '#' );
    if( false !== $pound_pos ) {
      # We gave a comment
      # Chop out the comment
      $webstamp_tagfiles_file_line = substr( $webstamp_tagfiles_file_line, 0, $pound_pos );
      $webstamp_tagfiles_file_line = trim( $webstamp_tagfiles_file_line );
    }
    # Check for a now blank line
    if( '' == $webstamp_tagfiles_file_line ) {
      # Ignore blank lines
      continue;
    }
    #echo "DEBUG: line: [${webstamp_tagfiles_file_line}]<br>\n";
    # Check for '^MAIN'
    if( '^MAIN' == $webstamp_tagfiles_file_line ) {
      # Never return '^MAIN' from this function
      #echo "DEBUG: Never return '^MAIN' from get_file_list()<br>\n";
      continue;
    }
    # Check for a readable tagfile
    if( false === is_readable( realpath( $directory . '/' . $webstamp_tagfiles_file_line ))) {
      # ignore unreadable tagfiles
      #echo "DEBUG: tagfile [${directory}/${webstamp_tagfiles_file_line}] not readable<br>\n";
      continue;
    }
    # Check for prefix
    if( '' != $prefix ) {
      # Prefix needed
      if( false === strpos( $webstamp_tagfiles_file_line, $prefix )) {
        # ignore lines when prefix wanted and not found
        #echo "DEBUG: prefix [${prefix}] wanted and not found in [${webstamp_tagfiles_file_line}]<br>\n";
        continue;
      }
    } else {
      # Prefix not wanted
      if( false !== strpos( $webstamp_tagfiles_file_line, '^' )) {
        # ignore lines when prefix [^] wanted and found
        #echo "DEBUG: prefix [^] not wanted and was found in [${webstamp_tagfiles_file_line}]<br>\n";
        continue;
      }
    }
    # Strip the '^' prefix if needed
    if( '^' == $prefix ) {
      $webstamp_tagfiles_file_line = str_replace( '^', '', $webstamp_tagfiles_file_line );
    }
    # Add the filename contained in the tagfiles file line to the file_list array
    #echo "DEBUG: Adding [${webstamp_tagfiles_file_line}] to the file_list array<br>\n";
    //It's stupid we have to build the array this way, but there IS a
    //bug in this version of PHP (4.0.3)
    $file_list = array_merge( $file_list, array( 0 => $webstamp_tagfiles_file_line ));
  }
  return $file_list;
}

function get_file_list_old( $directory, $prefix = '' ) {
  global $OURLIBS;

  $prefix = !empty( $prefix ) ? '\\'.$prefix : '';
  $d = dir( $directory );
  $file_list = array();
  while( $file = $d->read() ) {
    if( ereg( '^' . $prefix . '_?[A-Z][A-Za-z0-9_]*$', $file )
        && strcmp( $file, 'CVS' )  
        && strcmp( $file, '^MAIN' )
        && is_readable( realpath( $directory . '/' . $file ))) {
      //It's stupid we have to build the array this way, but there IS a
      //bug in this version of PHP (4.0.3)
      $file = str_replace( '^', '', $file );
      $file_list = array_merge( $file_list, array( 0 => $file ));
      // $file_list[] = $file;  //I'd much rather build the array this way.
    }
  }
  $d -> close();
  $d = dir( $OURLIBS );
  while( $file = $d->read() ) {
    if( ereg('^=_?[A-Z][A-Za-z0-9_]*$', $file) 
        && strcmp( $file, 'CVS' )  
        && strcmp( $file, '^MAIN' )
        && is_readable( realpath( $OURLIBS . '/' . $file ))) {
      // It's stupid we have to build the array this way, but there IS a
      // bug in this version of PHP (4.0.3)
      $file = str_replace( '^', '', $file );
      $file_list = array_merge( $file_list, array( 0 => $file ));
      // $file_list[] = $file;  //I'd much rather build the array this way.
    }
  }
  $d -> close();
  return $file_list;
}

function parse_dynamic_tag( $tag, $tagfile ) {
  global $ignore_http_get_vars, $ignore_http_post_vars;

  srand( (double) microtime() * 1000000 );
  $tagfile_contents = '';
  $funcname = "dynamic${tag}_" . rand( 0, 99999999 );
  $tagfile_contents .= @implode( '', file( $tagfile ));
  // Below, we don't use newlines so all the 'overhead' code generated by webstamp should hopefully not impact eval'd code line numbers anymore
  $funcdef_contents = "function $funcname() {";
  $funcdef_contents .= '  global $OURDIR, $OURCACHE, $OURDATA, $OURLIBS, $OURURL, $OURGFXURL;';
  $funcdef_contents .= '  global $ROOTDIR, $ROOTCACHE, $ROOTDATA, $ROOTLIBS, $ROOTURL, $ROOTGFXURL;';
  $funcdef_contents .= '  global $THISDIR, $THISURL;';
  if( !isset( $_GET ) && isset( $GLOBALS[ 'HTTP_GET_VARS' ] )) {
    $_GET = $GLOBALS[ 'HTTP_GET_VARS' ];
  }
  if( isset( $_GET )) {
    $funcdef_contents .= '  extract( $_GET );';
  } else if( !isset( $ignore_http_get_vars ) || $ignore_http_get_vars == false ) {
    echo "\$_GET (nor \$HTTP_GET_VARS) was not made available by\n";
    echo "PHP.  Webstamp expects it to be available for fields included in\n";
    echo "a GET method form.  If it's lack of availability is intentional,\n";
    echo "then set the \$ignore_http_get_vars variable in your root\n";
    echo "siteconfig.inc to true.\n";
    die();
  }
  if( !isset( $_POST ) && isset( $GLOBALS[ 'HTTP_POST_VARS' ] )) {
    $_POST = $GLOBALS[ 'HTTP_POST_VARS' ];
  }
  if( isset( $_POST )) {
    $funcdef_contents .= '  extract( $_POST );';
  } else if( !isset( $ignore_http_post_vars ) || $ignore_http_post_vars == false ) {
    echo "\$_POST (nor \$HTTP_POST_VARS) was not made available by PHP.  Webstamp\n";
    echo "expects it to be available for fields included in a POST method form. \n";
    echo "If it's lack of availability is intentional, then set the\n";
    echo "\$ignore_http_post_vars variable in your root siteconfig.inc to true.\n";
    die();
  }
  $funcdef_contents .= $tagfile_contents;
  $funcdef_contents .= "}\n";
  eval( $funcdef_contents );
  ob_start();
  eval( "\$result = $funcname();" );
  $file_contents = ob_get_contents();
  ob_end_clean();
  return( $file_contents );
}

function parse_file_array( $file_array, $workingset, $OURLIBS, $directory, $prefix = '' ) {
  if( count($file_array) > 0 ) {
    while( list($i, $tag) = each($file_array) ) {
      if( $tag[0] == '_' ) {
        $file_contents = parse_dynamic_tag( $tag, $directory . '/' . $prefix . $tag );
      } else if( $tag[0] == '=' && $tag[1] == '_' ) {
        $shavedtag = substr( $tag, 1, strlen( $tag ));
        $file_contents = parse_dynamic_tag( $shavedtag, $OURLIBS . '/' . $tag );
      } else if( $tag[0] == '=' ) {
        $file_contents = @implode('', file( $OURLIBS . '/' . $tag ));
      } else {
        $file_contents = @implode('', file( "$directory/$prefix$tag" ));
      }
      $search = '{' . $tag . '}';
      $workingset = str_replace( $search, $file_contents, $workingset );
    }
  }
  return $workingset;
}

function process( $OURLIBS, $directory, $workingset, $prefix = '' ) {
  $files = get_file_list( $directory, $prefix );
  $recursive_list = array();
  do {
    unset( $todo_list );
    $tags_list = get_needed_tags_list( $workingset );
    $tags_list = array_unique( $tags_list );
    $unfound_list = array_diff( $tags_list, $files );
    $tags_list = array_intersect( $tags_list, $files );
    while( list( $i, $tag ) = each( $tags_list )) {
      if( $tag[0] != '_' && $tag[0] != '=' ) {
        $file_contents = implode( '', file( $directory.'/'.$prefix.$tag ));
        if( strstr( $file_contents, '{' . $tag . '}' )) {
          $recursive_list = array_unique( array_merge( $recursive_list, array( 0 => $tag )));
        }
      }
    }
    $todo_list = array_diff( $tags_list, $recursive_list );
    $workingset = parse_file_array( $todo_list, $workingset, $OURLIBS, $directory, $prefix );
  } while( count( $todo_list ) > 0 );
  $workingset = parse_file_array( $recursive_list, $workingset, $OURLIBS, $directory, $prefix );
  unset( $unfound_list, $recursive_list, $todo_list );
  return $workingset;
}

function gzip_accepted( $ACCEPTED_ENCODINGS ) {
  if( strpos( $ACCEPTED_ENCODINGS, 'gzip' ) === false ) {
    return false;
  }
  if( strpos( $ACCEPTED_ENCODINGS, 'x-gzip' ) === false ) {
    $encoding = 'gzip';
  } else {
    $encoding = 'x-gzip';
  }
  return $encoding;
}

?>
